#include <QDebugStateSaver>

#include "triangle.h"

Triangle::Triangle() {}

Triangle::Triangle(const Triangle &other) {
    this->vertex_buffer_ = other.vertex_buffer_;
    this->u_ = other.u_;
    this->v_ = other.v_;
    this->w_ = other.w_;
}

Triangle::Triangle(const QVector<Vertex> &vertex_buffer, int u, int v, int w) {
    this->vertex_buffer_ = vertex_buffer;
    this->u_ = u;
    this->v_ = v;
    this->w_ = w;
}

Vertex Triangle::u() const {
    return vertex_buffer_[u_];
}

Vertex Triangle::v() const {
    return vertex_buffer_[v_];
}

Vertex Triangle::w() const {
    return vertex_buffer_[w_];
}

int Triangle::u_idx() const {
    return u_;
}

int Triangle::v_idx() const {
    return v_;
}

int Triangle::w_idx() const {
    return w_;
}

Triangle &Triangle::operator=(const Triangle &other) {
    this->vertex_buffer_ = other.vertex_buffer_;
    this->u_ = other.u_;
    this->v_ = other.v_;
    this->w_ = other.w_;
    return *this;
}

bool Triangle::operator==(const Triangle &other) const {
    return u_ == other.u_ && v_ == other.v_ && w_ == other.w_;
}

bool Triangle::operator<(const Triangle &other) const {
    if (u_ < other.u_) {
        return true;
    } else if (u_ > other.u_) {
        return false;
    }

    if (v_ < other.v_) {
        return true;
    } else if (v_ > other.v_) {
        return false;
    }

    if (w_ < other.w_) {
        return true;
    } else if (w_ > other.w_) {
        return false;
    }

    return false;
}

QDebug operator<<(QDebug d, const Triangle &triangle) {
    QDebugStateSaver saver(d);
    d.nospace() << "Triangle(" << triangle.u_idx() << ", " << triangle.v_idx() << ", " << triangle.w_idx() << "; " << triangle.u().pos << ", " << triangle.v().pos << ", " << triangle.w().pos << ")";
    return d;
}
